#version 150

#moj_import <matrix.glsl>

uniform sampler2D Sampler0;
uniform sampler2D Sampler1;

uniform float GameTime;
uniform int EndPortalLayers;

in vec4 texProj0;

const vec3[] GALACTIC_COLORS = vec3[](
    vec3(0.2, 0.0, 0.5),  // Roxo escuro
    vec3(0.4, 0.0, 0.8),  // Roxo médio
    vec3(0.6, 0.0, 1.0),  // Azul claro
    vec3(0.8, 0.2, 1.0),  // Azul com rosa
    vec3(1.0, 0.4, 1.0)   // Rosa claro
);

const mat4 SCALE_TRANSLATE = mat4(
    0.5, 0.0, 0.0, 0.25,
    0.0, 0.5, 0.0, 0.25,
    0.0, 0.0, 1.0, 0.0,
    0.0, 0.0, 0.0, 1.0
);

mat4 end_portal_layer(float layer) {
    mat4 translate = mat4(
        1.0, 0.0, 0.0, 17.0 / layer,
        0.0, 1.0, 0.0, (2.0 + layer / 1.5) * (GameTime * 1.5),
        0.0, 0.0, 1.0, 0.0,
        0.0, 0.0, 0.0, 1.0
    );

    // Aumenta a escala para tornar os pixels maiores
    mat2 rotate = mat2_rotate_z(radians((layer * layer * 2000.0 + layer * 5.0) * 1.0));
    mat2 scale = mat2((3.0 - layer / 2.0) * 2.0);  // Aumenta o fator de escala

    return mat4(scale * rotate) * translate * SCALE_TRANSLATE;
}

out vec4 fragColor;

void main() {
    vec3 color = textureProj(Sampler0, texProj0).rgb * GALACTIC_COLORS[0];
    for (int i = 0; i < EndPortalLayers; i++) {
        float phase = (GameTime + float(i) * 0.2) * 0.5;
        vec3 galacticColor = mix(GALACTIC_COLORS[0], GALACTIC_COLORS[4], phase);
        color += textureProj(Sampler1, texProj0 * end_portal_layer(float(i + 1))).rgb * galacticColor;
    }
    // Adiciona um brilho galáctico dinâmico
    float brightness = abs(sin(GameTime * 0.3)) * 0.6 + 0.4;
    fragColor = vec4(color * brightness, 1.0);
}
